! *******************************************************************
! * BESCHREIBUNG : NETSTERR (NETZ STATUS ERROR) ANALYSIERT DEN      *
! *                UEBERGEBENEN TCP/IP FEHLERCODE, UND GIBT DIESEN  *
! *                IM KLARTEXT AUS. DAZU WIRD VOM FEHLERCODE DAS    *
! *                MSB(MOST SIGNIFICANT BIT) ENTFERNT UND DANN ALS  * 
! *                FIXED(15) ZURUECKGEGEBEN.                        *
! *                AUSGEGEBEN WIRD DIE MELDUNG AUF DER JEWEILS      *
! *                ANGEGEBENEN DATION, HIER IM BEISPIEL TERM.       *
! *-----------------------------------------------------------------*
! * BEISPIEL : ...                                                  *
! *            SYSTEM;                                              *
! *               TERM: /TY;                                        *
! *            ...                                                  *
! *            PROBLEM;                                             *
! *            SPC TERM DATION INOUT ALPHIC;                        *
! *            ...                                                  *
! *            DCL ERROR FIXED(15);                                 *
! *            DCL NETST FIXED(15);                                 *
! *            ...                                                  *
! *            #INCLUDE NETSTERR.INC                                *
! *            ...                                                  *
! *            ERROR=NETSTERR(NETST,'0'B,TERM); ! OHNE AUSGABE      *
! *            ERROR=NETSTERR(NETST,'1'B,TERM); ! MIT AUSGABE       *
! *            ...                                                  *
! *                                                                 *
! *-----------------------------------------------------------------*
! * AUTOR            : LARS SCHREINER                               *
! * ERSTELLT         : 29.04.1998                                   *
! * LETZTE AENDERUNG : 07.05.1998                                   *
! *******************************************************************

NETSTERR: PROC(NETSTATUS FIXED(15),AUSGABE BIT(1),TY DATION OUT ALPHIC IDENT)
              RETURNS(FIXED(15));

                                    /* DAZU GEHOERENDER NETSTATUS (NETST):   */
DCL SOEGEN     INV FIXED INIT(  1); /* GENERIC ERROR           (NETST=-32767)*/
DCL SOEAIU     INV FIXED INIT(  2); /* ALREADY IN USE          (NETST=-32766)*/
DCL SOEINV     INV FIXED INIT(  3); /* INVALID HANDLE          (NETST=-32765)*/
DCL SOENOP     INV FIXED INIT(  4); /* SOCKET NOT OPEN         (NETST=-32764)*/
DCL SOEUPF     INV FIXED INIT(  5); /* UNKNOWN PROTOCOL FAMILY (NETST=-32763)*/
DCL SOEIST     INV FIXED INIT(  7); /* INVALID SOCKET TYPE     (NETST=-32761)*/
DCL SOEIPN     INV FIXED INIT(  8); /* INVALID PROTOCOL NUMBER (NETST=-32760)*/
DCL SOEIC      INV FIXED INIT(  9); /* INVALID COMMAND         (NETST=-32759)*/
DCL SOEACL     INV FIXED INIT( 10); /* ALREADY CLOSED          (NETST=-32758)*/
DCL SOEACO     INV FIXED INIT( 11); /* ALREADY CONNECTED       (NETST=-32757)*/
DCL SOEABO     INV FIXED INIT( 12); /* ALREADY BOUND           (NETST=-32756)*/
DCL SOEPRM     INV FIXED INIT( 13); /* PARAMETER ERROR         (NETST=-32755)*/
DCL SOENAB     INV FIXED INIT( 14); /* NO ADRESS BOUND         (NETST=-32754)*/
DCL SOEEOF     INV FIXED INIT( 15); /* END OF FILE             (NETST=-32753)*/
DCL SOEQLE     INV FIXED INIT( 16); /* QUEUE LENGTH EXCEEDED   (NETST=-32752)*/
DCL SOERST     INV FIXED INIT(256); /* RESET                   (NETST=-32512)*/
DCL SOETIM     INV FIXED INIT(257); /* TIMEOUT                 (NETST=-32511)*/
DCL SOEIDU     INV FIXED INIT(258); /* DESTINATION UNREACHABLE (NETST=-32710)*/

DCL VOIDPTR1 REF INV STRUCT[];    ! For text 1
DCL VOIDPTR2 REF INV STRUCT[];    ! For text 2
DCL C255PTR  REF INV CHAR(255);
DCL TXLEN FIXED;


IF NETSTATUS < 0 THEN
   NETSTATUS=TOFIXED(TOBIT(NETSTATUS) AND '7FFF'B4);

   IF AUSGABE THEN  ! WENN AUSGABE AUF '1' GESETZT IST DANN GIB 
                            ! FEHLER IM KLARTEXT AUS!
      CASE NETSTATUS

      ALT (SOEGEN) VOIDPTR1 = '1 = Generic error.';
                   VOIDPTR2 = 'No further info available.';
   
      ALT (SOEAIU) VOIDPTR1 = '2 = Already in use.';
                   VOIDPTR2 = 'If OPEN/BIND/CONNECT: socket/port/prot already in use.';
   
      ALT (SOEINV) VOIDPTR1 = '3 = Invalid Handle.';
                   VOIDPTR2 = 'Handle outside range 0 ... 31.';

      ALT (SOENOP) VOIDPTR1 = '4 = Socket not open.';
                   VOIDPTR2 = 'Socket was not opened before.';

      ALT (SOEUPF) VOIDPTR1 = '5 = Unknown protocol family.';
                   VOIDPTR2 = 'Unsupported prot-family with RESERVE.';
   
      ALT (SOEIST) VOIDPTR1 = '7 = Invalid socket type.';
                   VOIDPTR2 = 'Unsupported socket type with RESERVE.';

      ALT (SOEIPN) VOIDPTR1 = '8 = Invalid protocol number.';
                   VOIDPTR2 = 'Incompatible or unknown with RESERVE.';

      ALT (SOEIC)  VOIDPTR1 = '9 = Invalid command.';
                   VOIDPTR2 = 'Wrong socket state or unknown command.';

      ALT (SOEACL) VOIDPTR1 = '10 = Already closed.';
                   VOIDPTR2 = 'Socket closed before by SHUTDOWN or CLOSE.';

      ALT (SOEACO) VOIDPTR1 = '11 = Already connected.';
                   VOIDPTR2 = 'Socket is already bound by CONNECT.';
   
      ALT (SOEABO) VOIDPTR1 = '12 = Already bound.';
                   VOIDPTR2 = 'Socket is already bound by BIND.';

      ALT (SOEPRM) VOIDPTR1 = '13 = Parameter error.';
                   VOIDPTR2 = 'Parameters (Length etc.) invalid.';

      ALT (SOENAB) VOIDPTR1 = '14 = No address bound.';
                   VOIDPTR2 = 'Command refused: Socket not bound to an address.';
   
      ALT (SOEEOF) VOIDPTR1 = '15 = End of file.';
                   VOIDPTR2 = 'Remote host has already closed connection.';

      ALT (SOEQLE) VOIDPTR1 = '16 = Queue length exceeded.';
                   VOIDPTR2 = 'Command refused: Capacity of queue exceeded.';
   
      ALT (SOERST) VOIDPTR1 = '256 = Reset.';
                   VOIDPTR2 = 'Remote host did reset the connection.';
   
      ALT (SOETIM) VOIDPTR1 = '257 = Timeout.';
                   VOIDPTR2 = 'Due to exceeded time-limit connection is reset.';
   
      ALT (SOEIDU) VOIDPTR1 = '258 = Destination unreachable.';
                   VOIDPTR2 = 'Connection is reset, remote host unreachable.';

      OUT          VOIDPTR1 = NIL;
                   VOIDPTR2 = 'Error-number out of range, no info available.';

      FIN;

      IF VOIDPTR1 ISNT NIL THEN
        C255PTR = VOIDPTR1;
        TXLEN = INSTR(C255PTR,1,255,'.',1,1);
        PUT C255PTR TO TY BY A(TXLEN),SKIP;

      ELSE

        PUT NETSTATUS,' = ???' TO TY BY F(8),A,SKIP;
      FIN;

      C255PTR = VOIDPTR2;
      TXLEN = INSTR(C255PTR,1,255,'.',1,1);
      PUT C255PTR TO TY BY A(TXLEN),SKIP;

   FIN; /*IF AUSGABE THEN  */



   RETURN(NETSTATUS);

ELSE

   RETURN(0);

FIN; /* IF NETSTATUS < 0 THEN */

END; /* NETSTERR */
